package com.ozs.web.controller.accountmanagment;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.config.BaseConfig;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.ApiTokenUtils;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.ChineseToPingyin;
import com.ozs.common.utils.IdempotenceUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.config.ServerConfig;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseDeviceDynamicManagement;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.CameraTree;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.vo.BaseCameraManagementHomeVo;
import com.ozs.service.entity.vo.BaseCameraManagementVo;
import com.ozs.service.entity.vo.BaseCameraManagementVos;
import com.ozs.service.entity.vo.BaseCameraResVo;
import com.ozs.service.entity.vo.MsgAlarmResp;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.BaseDeviceDynamicManagementService;
import com.ozs.service.service.BaseRailwayManagementService;
import com.ozs.service.service.MsgAlarmService;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysDeptService;
import com.ozs.web.core.config.CaneraConfig;
import com.ozs.web.core.util.CameraUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.InputStream;
import java.text.Collator;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 相机台账管理表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-17
 */
@RestController
@RequestMapping("/service/baseCameraManagement")
@Slf4j
public class BaseCameraManagementController extends BaseController {
    @Resource
    BaseCameraManagementService baseCameraManagementService;
    @Resource
    BaseDeviceDynamicManagementService baseDeviceDynamicManagementService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private MsgAlarmService msgAlarmService;
    @Autowired
    private BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private ServerConfig serverConfig;
    @Autowired
    private CaneraConfig caneraConfig;
    @Autowired
    private IdempotenceUtils idempotenceUtils;
    @Autowired
    private DataScoreUtil dataScoreUtil;

    /**
     * 所有相机列表+权限
     */
    @ApiOperation(value = "相机台账管理")
    @GetMapping("/selectCameraAll")
    public AjaxResult selectCameraAll() {
        String userId = getUserId();
        List<BaseCameraManagementHomeVo> baseCameraManagements = baseCameraManagementService.selectCameraAll(userId);
        return AjaxResult.success(baseCameraManagements);
    }

    @PostMapping(value = "/homeGetInfo")
    @ApiOperation("根据相机ID、是否解除 获取相机信息、未解除信息")
    public AjaxResult homeGetInfo(@RequestBody BaseCameraManagementHomeVo baseCameraManagementHomeVo) {
        Integer isLock = baseCameraManagementHomeVo.getIsLock();
        Long id = baseCameraManagementHomeVo.getId();
        String cameraCode = baseCameraManagementHomeVo.getCameraCode();
        // isLock  报警是否解除 1已解除2未解除
        if (!StringUtils.isEmpty(isLock) && isLock == 1 && !StringUtils.isEmpty(id)) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            baseCameraManagement.setDeptName(sysDept.getDeptName());
            baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
            String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            baseCameraManagement.setInstallMiles(mils);
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            baseCameraManagement.setEndMiles(end);
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            baseCameraManagement.setBeginMiles(begin);
            return AjaxResult.success(baseCameraManagement);
        } else if (!StringUtils.isEmpty(isLock) && isLock == 2 && !StringUtils.isEmpty(cameraCode)) {
            MsgAlarm msgAlarm = msgAlarmService.selectByCameraCode(cameraCode);
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = begin + "-" + end;
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarmResp.setImageUrls(objects);
            return AjaxResult.success(msgAlarmResp);
        }
        return null;
    }

    /**
     * 相机台账管理分页
     *
     * @param baseCameraManagementVo
     * @return
     */
    @ApiOperation(value = "相机台账管理分页")
    @PostMapping("/list")
    public AjaxResult selectBaseAccountManagement(@RequestBody BaseCameraManagementVo baseCameraManagementVo) {
        baseCameraManagementVo= (BaseCameraManagementVo) dataScoreUtil.setDataScore(getUserId(),baseCameraManagementVo);
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getLineDir()) && baseCameraManagementVo.getLineDir() != 0) {
            lw.eq(BaseCameraManagement::getLineDir, baseCameraManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseCameraManagementVo.getRailwayCode())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseCameraManagementVo.getRailwayCode());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseCameraManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseCameraManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseCameraManagementVo.getEndMile());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getInstallMiles())) {
            int miles = AppendUtils.stringSplit(baseCameraManagementVo.getInstallMiles());
            lw.eq(BaseCameraManagement::getInstallMile, miles);
        }
        if(!baseCameraManagementVo.getDsFlay()){
         if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDsUserId())){
             lw.eq(BaseCameraManagement::getCreateBy, baseCameraManagementVo.getDsUserId()).or();
         }
         if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDsDeptId())){
             lw.eq(BaseCameraManagement::getDeptId,baseCameraManagementVo.getDsDeptId()).or();
         }
         if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDsDeptIds())){
             lw.in(BaseCameraManagement::getDeptId,baseCameraManagementVo.getDsDeptIds());
         }
        }
        IPage<BaseCameraManagement> page = baseCameraManagementService.page(new Page<>(baseCameraManagementVo.getPageNum(), baseCameraManagementVo.getPageSize()), lw);
        if (!ObjectUtils.isEmpty(page) && !ObjectUtils.isEmpty(page.getRecords())) {
            List<BaseCameraManagement> dto1 = page.getRecords().stream().map(o -> {
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(o.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, o.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                SysDept sysDept = deptService.selectDeptById(o.getDeptId());
                o.setDeptName(sysDept.getDeptName());
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                String mils = AppendUtils.stringAppend(o.getInstallMile());
                o.setInstallMiles(mils);
                String end = AppendUtils.stringAppend(o.getEndMile());
                o.setEndMiles(end);
                String begin = AppendUtils.stringAppend(o.getBeginMile());
                o.setBeginMiles(begin);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * 删除相机台账管理信息
     *
     * @param accountManagementIds
     * @return
     */
    @DeleteMapping("/{accountManagementIds}")
    @ApiOperation("删除相机台账管理信息")
    @Transactional
    public AjaxResult removeAccountManagement(@PathVariable List<Long> accountManagementIds) {
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        for (Long accountManagementId : accountManagementIds) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(accountManagementId);
            baseCameraManagementService.removeById(accountManagementId);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
            }
            baseDeviceDynamicManagementService.remove(queryWrapper);
        }
        return AjaxResult.success();
    }

    /**
     * 新增相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/saveAccountManagement")
    @ApiOperation("新增相机台账管理信息")
    @Transactional
    public AjaxResult saveAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        String idempotence = idempotenceUtils.getIdempotence(baseCameraManagement.getCameraCode());
        if (StringUtils.isNotEmpty(idempotence)) {
            return error("你的网络卡顿,请不要一直添加!!!");
        }
        idempotenceUtils.createIdempotence(baseCameraManagement.getCameraCode());
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            lw.eq(BaseCameraManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(lw);
        if (!ObjectUtils.isEmpty(cameraManagement)) {
            return error("相机编码已存在,不能重复添加");
        }
        int miles = AppendUtils.stringSplit(baseCameraManagement.getBeginMiles());
        baseCameraManagement.setBeginMile(miles);
        int install = AppendUtils.stringSplit(baseCameraManagement.getInstallMiles());
        baseCameraManagement.setInstallMile(install);
        int end = AppendUtils.stringSplit(baseCameraManagement.getEndMiles());
        baseCameraManagement.setEndMile(end);
        baseCameraManagement.setCreateBy(getUserId());
        baseCameraManagement.setUpdateBy(getUserId());
        BaseDeviceDynamicManagement baseDynamicManagement = new BaseDeviceDynamicManagement();
        baseDynamicManagement.setCameraCode(baseCameraManagement.getCameraCode());
        baseDynamicManagement.setUpdateBy(getUserId());
        baseDynamicManagement.setCreateBy(getUserId());
        String start = HttpUtils.sendGet(CameraUtil.startRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
        if (start.startsWith(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel())) {
            baseCameraManagement.setStartRecording(1);
            baseCameraManagement.setTaskId(start);
        } else {
            baseCameraManagement.setStartRecording(2);
        }
        if (baseCameraManagementService.save(baseCameraManagement) && baseDeviceDynamicManagementService.save(baseDynamicManagement)) {
            idempotenceUtils.deleteIdempotence(baseCameraManagement.getCameraCode());
            return AjaxResult.success();
        }
        idempotenceUtils.deleteIdempotence(baseCameraManagement.getCameraCode());
        return AjaxResult.error();
    }

    /**
     * 修改相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PutMapping("/editAccountManagement")
    @ApiOperation("修改相机台账管理信息")
    public AjaxResult editAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            lw.eq(BaseCameraManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(lw);
        if (!ObjectUtils.isEmpty(cameraManagement) && !cameraManagement.getId().equals(baseCameraManagement.getId())) {
            return error("相机编码已存在,不能进行修改操作");
        }
        int miles = AppendUtils.stringSplit(baseCameraManagement.getBeginMiles());
        baseCameraManagement.setBeginMile(miles);
        int install = AppendUtils.stringSplit(baseCameraManagement.getInstallMiles());
        baseCameraManagement.setInstallMile(install);
        int end = AppendUtils.stringSplit(baseCameraManagement.getEndMiles());
        baseCameraManagement.setEndMile(end);
        baseCameraManagement.setUpdateBy(getUserId());
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseDeviceDynamicManagement one = baseDeviceDynamicManagementService.getOne(queryWrapper);
        one.setCameraCode(baseCameraManagement.getCameraCode());
        if (baseCameraManagementService.updateById(baseCameraManagement) && baseDeviceDynamicManagementService.updateById(one)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据相机ID获取相机台账管理信息")
    public AjaxResult getInfo(@PathVariable Long id) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
        LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
        SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
        baseCameraManagement.setDeptName(sysDept.getDeptName());
        baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
        String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        baseCameraManagement.setInstallMiles(mils);
        String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
        baseCameraManagement.setEndMiles(end);
        String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
        baseCameraManagement.setBeginMiles(begin);
        return AjaxResult.success(baseCameraManagement);
    }

    @GetMapping("/getDate/{cameraCode}")
    @ApiOperation("视频上方显示的数据")
    public AjaxResult getDate(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
        LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
        if (ObjectUtils.isEmpty(baseRailwayManagement)){
            return error("线路表中没有该线路编码:"+baseCameraManagement.getRailwayCode()+"的数据");
        }
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseDeviceDynamicManagement baseDeviceDynamicManagement = baseDeviceDynamicManagementService.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(baseRailwayManagement)){
            return error("设备动态管理表中没有该相机编码:"+baseCameraManagement.getCameraCode()+"的数据");
        }
        List<Object> list = new ArrayList<>();
        list.add(baseRailwayManagement.getRailwayName());
        list.add(AppendUtils.stringAppend(baseCameraManagement.getInstallMile()));
        list.add(baseCameraManagement.getLineDir());
        list.add(baseCameraManagement.getCameraCode());
        list.add(baseDeviceDynamicManagement.getElectricity());
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format1 = dateFormat1.format(new Date());
        list.add(format1);
        return AjaxResult.success(list);
    }

    /**
     * 台账管理导入
     *
     * @param file
     * @param updateSupport
     * @return
     * @throws Exception
     */
    @ApiOperation("相机台账管理信息导入")
    @PostMapping("/importBaseAccountManagement")
    public AjaxResult importBaseAccountManagement(MultipartFile file, boolean updateSupport) throws Exception {
        ExcelUtil<BaseCameraManagementVos> util = new ExcelUtil<BaseCameraManagementVos>(BaseCameraManagementVos.class);
        List<BaseCameraManagementVos> accountManageList = util.importExcel(file.getInputStream());
        String message = baseCameraManagementService.importBaseAccountManagement(accountManageList, updateSupport, getUserId());
        return AjaxResult.success(message);
    }

    /**
     * 导出台账管理
     *
     * @param response
     */
    @ApiOperation("导出相机台账管理信息")
    @PostMapping("/exportBaseAccountManagement")
    public void exportBaseAccountManagement(HttpServletResponse response, @RequestBody BaseCameraManagementVo baseAccountManagementVo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getLineDir())) {
            lw.eq(BaseCameraManagement::getLineDir, baseAccountManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseAccountManagementVo.getRailwayCode())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseAccountManagementVo.getRailwayCode());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseAccountManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseAccountManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseAccountManagementVo.getEndMile());
        }
        List<BaseCameraManagement> list = baseCameraManagementService.list(lw);
        for (BaseCameraManagement baseCameraManagement : list) {
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            baseCameraManagement.setDeptName(sysDept.getDeptName());
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
            String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            baseCameraManagement.setInstallMiles(mils);
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            baseCameraManagement.setEndMiles(end);
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            baseCameraManagement.setBeginMiles(begin);
        }
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<>(BaseCameraManagement.class);
        util.exportExcel(response, list, "台账管理相机数据");
    }


    @ApiOperation("相机台账管理信息模板下载")
    @GetMapping("/downloaExcel")
    public void downloadZip(HttpServletResponse response) {
        try {
            InputStream resourceAsStream = this.getClass().getResourceAsStream("/template/baseCameraManagement.xlsx");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, "相机台账管理信息模板.xlsx");
            FileUtils.writeBytesByInput(resourceAsStream, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    @ApiOperation("查询录像")
    @GetMapping("/api/records/{cameraCode}")
    public AjaxResult apiRecords(@PathVariable @NotNull(message = "相机编码不能为空") String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel()));
    }

    @ApiOperation("查询录像")
    @GetMapping("/api/records/{cameraCode}/{flay}")
    public AjaxResult apiRecords(@PathVariable @NotNull(message = "相机编码不能为空") String cameraCode,
                                 @PathVariable @NotNull(message = "条件不能为空") boolean flay) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel(), flay));
    }

    @ApiOperation("批量查询录像")
    @GetMapping("/api/recordsList")
    public AjaxResult apiRecordsList(@RequestBody
                                     @NotNull(message = "相机编码不能为空")
                                             List<String> cameraCodes) {
        List<String> list = new ArrayList<>();
        for (String cameraCode : cameraCodes) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
            BaseCameraManagement one = baseCameraManagementService.getOne(lw);
            if (ObjectUtils.isEmpty(one)) {
                throw new BaseException("相机编号【" + cameraCode + "】不存在");
            }
            String playFlv = null;
            if (list.size() > 5) {
                playFlv = CameraUtil.getPlayFlv(cameraCode, one.getChannel(), false);
            } else {
                playFlv = CameraUtil.getPlayFlv(cameraCode, one.getChannel());
            }
            list.add(playFlv);
        }

        return new AjaxResult(200, "ok", list);
    }


    @ApiOperation("播放回放-废弃")
    @PostMapping("/api/invite")
    public AjaxResult apiInvite(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        // CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime(), caneraConfig.getMappingUrl())
        return success();
    }

    @ApiOperation("播放回放")
    @PostMapping("/api/play")
    public AjaxResult play(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        String path = CameraUtil.historyPlayListStr(one.getChannel(), vo.getStartTime(), vo.getEntTime());
        if (org.apache.commons.lang3.StringUtils.isBlank(path)) {
            throw new BaseException("当前相机无视频录像");
        }
        return new AjaxResult(200, "ok", serverConfig.getUrl() + path);
//        return success(CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime()));
    }

    @ApiOperation("视频服务设备数量")
    @GetMapping("/totalNumberOfDevices")
    public AjaxResult totalNumberOfDevices() {
        long count = baseDeviceDynamicManagementService.count();
        return success(count);
    }

    @ApiOperation("视频服务树形结构")
    @GetMapping("/cameraTree")
    public AjaxResult cameraTree(@RequestParam(value = "name", required = false) String name) {
        CameraTree cameraTree = baseDeviceDynamicManagementService.cameraTree(name);
        List<CameraTree> children = cameraTree.getChildren();
        children.sort(new Comparator<CameraTree>() {
            @Override
            public int compare(CameraTree o1, CameraTree o2) {
                String name1 = ChineseToPingyin.convertHanziToPinyin(o1.getName());
                String substring1 = name1.substring(0, 1);
                String name2 = ChineseToPingyin.convertHanziToPinyin(o2.getName());
                String substring2 = name2.substring(0, 1);
                return substring1.compareTo(substring2);
            }
        });


        for (CameraTree child : children) {
            List<CameraTree> children1 = child.getChildren();
            children1.sort(new Comparator<CameraTree>() {
                @Override
                public int compare(CameraTree o1, CameraTree o2) {
                    int i = o1.getMileage() - o2.getMileage();
                    if (i == 0) {
                        String[] split = o1.getName().split("-");
                        return "上行".equals(split[1]) ? -1 : 1;
                    } else {
                        return i;
                    }
                }
            });
            for (CameraTree tree : children1) {
                List<CameraTree> children2 = tree.getChildren();
                if (children2.size() < 2) {
                    String code = children2.get(0).getName();
                    tree.setCode(code);
                    children2.remove(0);
                }
            }
        }
        return success(cameraTree);
    }

    @ApiOperation("关闭视频录制")
    @GetMapping("/endRecording/{id}")
    public AjaxResult endRecording(@PathVariable Long id) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
        log.info("endRecording------------" + baseCameraManagement.getTaskId());
        String start = HttpUtils.sendGet(CameraUtil.endRecording(baseCameraManagement.getTaskId()));
        log.info("endRecording------------" + start);
        if ("success".equals(start)) {
            baseCameraManagement.setStartRecording(2);
            baseCameraManagement.setTaskId("");
            baseCameraManagement.setUpdateBy(getUserId());
            if (baseCameraManagementService.updateById(baseCameraManagement)) {
                return success();
            }
        }
        return error("关闭视频录制失败");
    }


    @ApiOperation("开启视频录制")
    @GetMapping("/startRecording/{id}")
    public AjaxResult startRecording(@PathVariable Long id) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
        log.info("startRecording------CameraCode------" + baseCameraManagement.getCameraCode());
        log.info("startRecording------Channel------" + baseCameraManagement.getChannel());
        String start = HttpUtils.sendGet(CameraUtil.startRecording(baseCameraManagement.getCameraCode(), baseCameraManagement.getChannel()));
        log.info("startRecording------------" + start);
        if (start.startsWith(baseCameraManagement.getCameraCode() + "/" + baseCameraManagement.getChannel())) {
            log.info("startRecording 返回成功！！1");
            baseCameraManagement.setStartRecording(1);
            baseCameraManagement.setTaskId(start);
            baseCameraManagement.setUpdateBy(getUserId());
        } else {
            return error("开启视频录制失败");
        }
        if (baseCameraManagementService.updateById(baseCameraManagement)) {
            return success();
        } else {
            return error("开启视频录制失败");
        }
    }

    @GetMapping(value = "/getAlarm/{cameraCode}")
    @ApiOperation("根据视频ID获取详细信息")
    public AjaxResult getAlarm(@PathVariable String cameraCode) {
        LambdaQueryWrapper<MsgAlarm> queryWrapper = new LambdaQueryWrapper<MsgAlarm>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            queryWrapper.eq(MsgAlarm::getCameraCode, cameraCode);
            queryWrapper.orderByDesc(MsgAlarm::getCreateTime);
        }
        List<MsgAlarm> msgAlarmList = msgAlarmService.list(queryWrapper);
        if (msgAlarmList.size() > 0) {
            MsgAlarm msgAlarm = msgAlarmList.get(0);
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(cameraCode)) {
                lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = begin + end;
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            msgAlarmResp.setLineDir(baseCameraManagement.getLineDir());
            msgAlarmResp.setAlarmMile(baseCameraManagement.getInstallMile().toString());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }
}

